<?php
/**
 * @file
 * Classes and related functions for survey and survey response entity types
 */

/**
 * Main class for survey entities.
 */
class SurveyEntity extends Entity {

  public function __construct(array $values = array(), $entityType = NULL) {
    parent::__construct($values, 'survey');
  }

  /**
   * Specifies the default uri, which is picked up by uri() by default.
   */
  protected function defaultURI() {
    return array('path' => 'survey/' . $this->identifier());
  }

}

/**
 * Load a single survey entity from its ID
 *
 * @param $pid
 *   The entity ID.
 */
function survey_load($pid) {
  $entities = entity_load('survey', array($pid));
  if (count($entities)) {
    return array_shift($entities);
  }
  return FALSE;
}

/**
 * Main class for survey response entities.
 */
class SurveyResponseEntity extends Entity {

  public function __construct(array $values = array(), $entityType = NULL) {
    parent::__construct($values, 'survey_response');
  }

  /**
   * Specifies the default uri, which is picked up by uri() by default.
   */
  protected function defaultURI() {
    return array('path' => 'survey-response/' . $this->identifier());
  }

}

/**
 * Load a single survey response entity from its ID
 *
 * @param $pid
 *   The entity ID.
 */
function survey_response_load($pid) {
  $entities = entity_load('survey_response', array($pid));
  if (count($entities)) {
    return array_shift($entities);
  }
  return FALSE;
}

/**
 * Save survey response
 */
function survey_response_save($survey_id, $values) {

  global $user;

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.question');

  $survey = survey_load($survey_id);
  if (!$survey) {
    watchdog('survey_builder', 'Invalid survey: %id',
      array('%id' => $survey_id), WATCHDOG_ERROR);
    return FALSE;
  }

  $response = new SurveyResponseEntity();
  $response->survey_id = $survey_id;
  $response->uid = $user->uid;
  $response->created = time();
  $response->save();

  $success = survey_response_questions_save($survey->fields, $values, $response->pid);
  if (!$success) {
    return FALSE;
  }

  return $response->pid;

}

function survey_response_questions_save($questions, $values, $survey_response_id) {

  foreach ($questions as $key => $field) {

    if (strpos($key, '#') === 0) {
      continue;
    }
    elseif (isset($field['#type']) && $field['#type'] == 'fieldset') {

      $success = survey_response_questions_save($field, $values, $survey_response_id);
      if (!$success) {
        return FALSE;
      }

    }
    else {

      if (!isset($values[$key])) {
        $values[$key] = '';
      }

      $success = question_response_save($survey_response_id, $field['#question_id'], $values[$key]);
      if (!$success) {
        watchdog('survey_builder', 'Unable to save response for question: %id',
          array('%id' => $field['#question_id']), WATCHDOG_ERROR);
        return FALSE;
      }

    }

  }

  return TRUE;

}
